"""Utility functions for quantum-AI integration."""

from quantum_ai.utils.input_adapters import (
    adapt_quantum_input_for_model,
    standardize_quantum_input,
)

__all__ = ["adapt_quantum_input_for_model", "standardize_quantum_input"]
